from django import forms
from django.contrib.auth.decorators import login_required
from django.shortcuts import render_to_response
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext as _
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect, HttpResponseForbidden
from django.views.generic.list_detail import object_detail, object_list
from threadedcomments.models import ThreadedComment
from threadedcomments.forms import ThreadedCommentForm
from threadedcomments.views import free_comment
from xerxes.influences.models import Character
from xerxes.influences.models import Influence
from xerxes.tools.forms import ContextModelForm

#
# A few comment form classes, to handle the various cases (staff/no staff,
# reply to public/private post)
# 
# It is probably possible to reduce this mess a bit using metaclasses, but I
# didn't get this to work yet.
# 
class InfluenceCommentForm(ThreadedCommentForm):
    def __init__(self, *args, **kwargs):
        super(InfluenceCommentForm, self).__init__(*args, **kwargs)

    class Meta(ThreadedCommentForm.Meta):
        exclude = ('markup', )

class AdminInfluenceCommentForm(ThreadedCommentForm):
    def __init__(self, *args, **kwargs):
        super(AdminInfluenceCommentForm, self).__init__(*args, **kwargs)

    class Meta(ThreadedCommentForm.Meta):
        fields = ThreadedCommentForm.Meta.fields + ('is_public',)

class AdminPrivateInfluenceCommentForm(ThreadedCommentForm):
    def __init__(self, *args, **kwargs):
        super(AdminPrivateInfluenceCommentForm, self).__init__(*args, **kwargs)
        self.instance.is_public = False

def get_influence_comment_form(is_staff, reply_to):
    """ Gets the form class that a user can use to reply to the given comment.
    reply_to can be None, in which case the form class for a new comment is
    returned. """
    allow_markup = allow_private = is_staff
    if reply_to:
        allow_public = reply_to.is_public
    else:
        allow_public = True
    return _get_influence_comment_form(allow_markup, allow_public, allow_private)
    
def _get_influence_comment_form(allow_markup, allow_public, allow_private):
    """ Internal wrapper that selects the right form class depending on the
    given options. Should not be called directly, call
    get_influence_comment_form instead. """
    if allow_markup and allow_public and allow_private:
        return AdminInfluenceCommentForm
    elif allow_markup and not allow_public and allow_private:
        return AdminPrivateInfluenceCommentForm
    elif not allow_markup and allow_public and not allow_private:
        return InfluenceCommentForm
    else:
        raise Exception("Unsupported configuration")

class InfluenceForm(ContextModelForm):
    class Meta:
        model = Influence
        fields = ('character', 'contact', 'summary', 'description')

class CharacterForm(ContextModelForm):
    class Meta:
        model = Character
        fields = ('name')

@login_required
def add_influence(request, character_id=None):
    initial = {}
    # Get the current user's characters
    chars = request.user.character_set.all()

    # If a character_id was specified in the url, or there is only one
    # character, preselect it.
    if (character_id):
        initial['character'] = character_id
    elif (chars.count() == 1):
        initial['character'] = chars[0].id


    f = InfluenceForm(request=request, initial=initial)

    # Only allow characters of the current user. Putting this here also
    # ensures that a form will not validate when any other choice was
    # selected (perhaps through URL crafting).
    f.fields['character']._set_queryset(chars)

    if (f.is_valid()):
        # The form was submitted, let's save it.
        influence = f.save()
        # Redirect to the just saved influence
        return HttpResponseRedirect(reverse('influences_influence_detail', args=[influence.id]))

    return render_to_response('influences/add_influence.html', {'form' : f}, RequestContext(request))

@login_required
def add_character(request):
    f = CharacterForm(request=request)
    if (f.is_valid()):
        character = f.save(commit=False)
        character.player = request.user
        character.save()
        return HttpResponseRedirect(reverse('influences_character_detail', args=[character.id]))

    return render_to_response('influences/add_character.html', {'form' : f}, RequestContext(request))

@login_required
def index(request):
    # Only show this player's characters and influences
    characters = request.user.character_set.all()
    influences = Influence.objects.filter(character__player=request.user)
    return render_to_response('influences/index.html', {'characters' : characters, 'influences' : influences}, RequestContext(request))

#
# The views below are very similar to django's generic views (in fact,
# they used to be generic views before). However, since they all depend
# on the currently logged in user (for limiting the show list or
# performing access control), we won't actually use the generic views
# here.

@login_required
def character_list(request):
    # Only show this player's characters
    os = request.user.character_set.all()
    return render_to_response('influences/character_list.html', {'object_list' : os}, RequestContext(request))

@login_required
def character_detail(request, object_id):
    o = Character.objects.get(pk=object_id)
    # Don't show other player's characters
    if (o.player != request.user):
        return HttpResponseForbidden("Forbidden -- Trying to view somebody else's character")
    return render_to_response('influences/character_detail.html', {'object' : o}, RequestContext(request))

@login_required
def influence_list(request):
    # Only show this player's influences
    os = Influence.objects.filter(character__player=request.user)
    return render_to_response('influences/influence_list.html', {'object_list' : os}, RequestContext(request))

def quote_reply(comment):
    return "\n".join(["> " + l for l in comment.comment.split("\n")])

@login_required
def influence_detail(request, object_id):

    o = Influence.objects.get(pk=object_id)
    # Don't show other player's influences
    if (o.character.player != request.user):
        return HttpResponseForbidden("Forbidden -- Trying to view influences of somebody else's character")

    # Show all comments to staff, but only public comments to other
    # users
    if request.user.is_staff:
        comments = ThreadedComment.objects.get_tree(o)
    else:
        comments = ThreadedComment.public.get_tree(o)

    # Annotate each comment with a proper reply form
    for comment in comments:
        initial = { 'comment' : quote_reply(comment) }
        comment.reply_form = get_influence_comment_form(request.user.is_staff, comment)(initial=initial)
    
    context  = {
        'object' : o,
        'comments' : comments,
        'comment_form' : get_influence_comment_form(request.user.is_staff, None)()
    }
    return render_to_response('influences/influence_detail.html', context, RequestContext(request))

@login_required
def influence_comment(request, edit_id=None, *args, **kwargs):
    # Add the content_type, since we don't put in in the url explicitly
    kwargs['content_type'] = ContentType.objects.get_for_model(Influence).id
    # Find the comment to which we're replying, so we can get the right form for it.
    if edit_id:
        reply_to = get_object_or_404(ThreadedComment, id=edit_id)
    else:
        reply_to = None
    # Find the right form class
    kwargs['form_class']   = get_influence_comment_form(request.user.is_staff, reply_to)
    # Override the model, so we don't get a free comment, but a normal
    # one. We can't use threadedcomments' comment view for that, since
    # that hardcodes the form_class.
    kwargs['model'] = ThreadedComment
    return free_comment(request, edit_id=edit_id, *args, **kwargs)

# vim: set sts=4 sw=4 expandtab:
